unit sg4;

(* 
 * Delphi Sample Code provided by BITWISE MAGAZINE
 * 
 *      http://www.bitwisemag.com
 *
 * This code and information is provided "as is" without warranty of
 * any kind, either expressed or implied, including but not limited
 * to the implied warranties of merchantability and/or fitness for a
 * particular purpose.
 *
 * You may use and modify this code for your personal use only.
 * You may also redistribute this code providing that
 * a) No fee is charged and b) This copyright notice
 * is retained in the source code.
 *
 * You may not use this code for commercial purposes
 * without the express permission of the copyright holder
 *
 * Copyright (c) 2005 Huw Collingbourne 
 *
 *)

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, Menus, ExtCtrls;


Type
  GrabType = (GTSCREEN,GTWINDOW,GTCLIENT);

type
  TForm1 = class(TForm)
    MainMenu1: TMainMenu;
    GrabMenu: TMenuItem;
    GrabScreen: TMenuItem;
    GrabWindow: TMenuItem;
    Image1: TImage;
    GrabClientArea: TMenuItem;
    ViewMenu: TMenuItem;
    FullScreenMenuItem: TMenuItem;
    FileMenu: TMenuItem;
    ClearMenuItem: TMenuItem;
    N1: TMenuItem;
    ExitMenuItem: TMenuItem;
    GrabSelectiedArea: TMenuItem;
    procedure GrabWindowClick(Sender: TObject);
    procedure GrabScreenClick(Sender: TObject);
    procedure GrabClientAreaClick(Sender: TObject);
    procedure FullScreenMenuItemClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure ClearMenuItemClick(Sender: TObject);
    procedure ExitMenuItemClick(Sender: TObject);
    procedure Image1MouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure Image1MouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure Image1MouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure GrabSelectedAreaClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    FmT,FmL,FmH,FmW : integer;       // Save Form dimensions
    SelX,SelY,SelX1,SelY1 : integer; // Selection rectangle
    DoSelect : boolean;
    FormIsFullScreen : boolean;
    procedure SaveFormSize;
    procedure RestoreFormSize;
    procedure MaximizeForm;
    procedure RestoreForm;
    procedure DrawCursor(bm : TBitmap; OffsetX, OffsetY : integer);
    procedure Grab(bm: TBitMap; gt : GrabType);
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}


procedure TForm1.DrawCursor(bm : TBitmap; OffsetX, OffsetY : integer);
// Two OffSet Parameters will be 0 for full screen or positive offets
// for a window grab
var
  cursorPos : TPoint;
  CursorInf: TCursorInfo;
  Icon: TIcon;
  IconInf: TIconInfo;
begin
  Icon := TIcon.Create;
  try
    CursorInf.cbSize := SizeOf(TCursorInfo);
    if GetCursorInfo(CursorInf) then
      if CursorInf.Flags = CURSOR_SHOWING then
      begin
        Icon.Handle := CursorInf.hCursor;
        if GetIconInfo(Icon.Handle, IconInf) then
            DrawIcon( bm.Canvas.Handle,
                      CursorInf.ptScreenPos.x - (IconInf.xHotspot + OffSetX),
                      CursorInf.ptScreenPos.y - (IconInf.yHotspot+ OffSetY),
                      Icon.Handle);
      end;
  finally
    Icon.Free;
  end;
end;


procedure TForm1.Grab(bm: TBitMap; gt : GrabType);
var
  DestRect, SourceRect: TRect;
  h: THandle;
  hdcSrc : THandle;
  pt : TPoint;
begin
  case(gt) of
    GTWINDOW, GTCLIENT : h := GetForeGroundWindow;
    GTSCREEN : h := GetDesktopWindow;
  else h := 0;
  end;
  if h <> 0 then
  begin
    try
      if gt = GTCLIENT then
      begin
        hdcSrc := GetDC(h); // use this for ClientRect
        Windows.GetClientRect(h,SourceRect);
      end
      else
      begin
          hdcSrc := GetWindowDC(h);
          GetWindowRect(h, SourceRect);
      end;
        bm.Width  := SourceRect.Right - SourceRect.Left;
        bm.Height := SourceRect.Bottom - SourceRect.Top;
        DestRect := Rect(0, 0, SourceRect.Right - SourceRect.Left, SourceRect.Bottom - SourceRect.Top);
          StretchBlt(bm.Canvas.Handle, 0, 0, bm.Width,
            bm.Height, hdcSrc,
            0,0,SourceRect.Right - SourceRect.Left,
            SourceRect.Bottom - SourceRect.Top,
            SRCCOPY);
        if gt = GTCLIENT then
        begin
           pt.X :=SourceRect.Left;
           pt.Y :=SourceRect.Top;
           // call Windows.ClientToScreen() to translate X and Y
           Windows.ClientToScreen(h, pt);
           DrawCursor(bm,pt.X, pt.Y);
        end
        //
        else
          DrawCursor(bm,SourceRect.Left, SourceRect.Top);
    finally
      ReleaseDC(0, hdcSrc);
    end;
  end;
end;


procedure TForm1.MaximizeForm;
begin
     SaveFormSize; // save and restore form size (lost when BorderStyle=bNone)
   //  Form1.WindowState := wsMaximized;  // Maximized state isn't preserved on Form1.Show
     Form1.SetBounds(0,0,Screen.Width,Screen.Height);
     Form1.BorderStyle := bsNone;
     Form1.Menu := nil;    // remove menu
     FormIsFullScreen := true;
end;

procedure TForm1.RestoreForm;
begin
      Form1.WindowState := wsNormal;
      Form1.BorderStyle := bsSizeable;
      Form1.FormStyle := fsNormal;
      RestoreFormSize;
      Form1.Menu := MainMenu1;  // add menu
      FormIsFullScreen := false;
end;

procedure TForm1.FullScreenMenuItemClick(Sender: TObject);
begin
   if FormIsFullScreen then
      RestoreForm
   else
      MaximizeForm;
end;

procedure TForm1.SaveFormSize;
begin
  FmT := Form1.Top;
  FmL := Form1.Left;
  FmH := Form1.Height;
  FmW := Form1.Width;
end;

procedure TForm1.RestoreFormSize;
begin
  Form1.Top := FmT;
  Form1.Left := FmL;
  Form1.Height := FmH;
  Form1.Width := FmW;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
   SaveFormSize;
   SelX := 0;
   SelY := 0;
   SelX1 := 0;
   SelY1 := 0;
   DoSelect := False;
   FormIsFullScreen := false;
   Form1.DoubleBuffered := true;  // avoids flicker
end;

procedure TForm1.FormKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
   if Key = 27 then // Escape
   begin
      if FormIsFullScreen then
          RestoreForm;
      DoSelect := false;
   end;
end;

procedure TForm1.ClearMenuItemClick(Sender: TObject);
begin
   Image1.Picture := nil;
end;

procedure TForm1.ExitMenuItemClick(Sender: TObject);
begin
   Close;
end;

procedure TForm1.Image1MouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
   if DoSelect then begin
     SelX := X;
     SelY := Y;
     SelX1 := X;
     SelY1 := Y;
     with Image1.Canvas do
     begin                    // Options shown in comments
        Pen.Width := 1;      // 2; // use with solid pen style
        Pen.Style := psDashDotDot; // psSolid;
        Pen.Mode := pmNotXOR; // pmXor;
        Brush.Style := bsClear;
        Pen.Color := clBlue; // clYellow;
     end;
   end;
end;

procedure TForm1.Image1MouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
  DestRect, SourceRect : TRect;
begin

  if DoSelect then begin
    Image1.Canvas.Rectangle(SelX,SelY,SelX1,SelY1);   // invert selection rect
    // Change coordinates if the user has drawn a rectangle 'backwards'
    // i.e. to left or above starting point, SelX,SelY
    if X <= SelX then
    begin
      SelX1 := SelX;
      SelX := X;
    end
    else
      SelX1 := X;

    if Y <= SelY then
    begin
      SelY1 := SelY;
      SelY := Y;
    end
    else
      SelY1 := Y;

    Image1.Canvas.Pen.Mode := pmCopy;
    SourceRect := Rect(SelX,SelY,SelX1,SelY1);
    DestRect := Rect(0,0,SelX1-SelX,SelY1-SelY);
    Image1.Canvas.CopyRect(DestRect,Image1.Canvas,SourceRect);
    Image1.Picture.Bitmap.Height := SelY1-SelY;
    Image1.Picture.Bitmap.Width := SelX1-SelX;
    Image1.SetBounds(0,0,SelX1-SelX,SelY1-SelY);
    DoSelect := false;
    if FormIsFullScreen then
      RestoreForm;
  end;
end;

procedure TForm1.Image1MouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
begin
   if DoSelect then begin
     if ssLeft in Shift then
     begin
      Image1.Canvas.Rectangle(SelX,SelY,SelX1,SelY1); // invert selection rect
      SelX1 := X;
      SelY1 := Y;
      Image1.Canvas.Rectangle(SelX,SelY,SelX1,SelY1); // draw new selection rect
     end;
   end;
end;

procedure TForm1.GrabScreenClick(Sender: TObject);
begin
   Form1.Hide;
   Sleep(50); // make sure the form is hidden before capture
   Grab(image1.Picture.Bitmap, GTSCREEN);
   image1.Width := image1.Picture.Bitmap.Width;
   image1.Height := image1.Picture.Bitmap.Height;
   Form1.Show;
end;


procedure TForm1.GrabWindowClick(Sender: TObject);
begin
   Form1.Hide;
   Sleep(5000);
   Grab(image1.Picture.Bitmap, GTWINDOW);
   image1.Width := image1.Picture.Bitmap.Width;
   image1.Height := image1.Picture.Bitmap.Height;
   Form1.Show;
end;

procedure TForm1.GrabClientAreaClick(Sender: TObject);
begin
   Form1.Hide;
   Sleep(5000);
   Grab(image1.Picture.Bitmap, GTCLIENT);
   image1.Width := image1.Picture.Bitmap.Width;
   image1.Height := image1.Picture.Bitmap.Height;
   Form1.Show;
end;


procedure TForm1.GrabSelectedAreaClick(Sender: TObject);
begin
   Form1.Hide;
   Sleep(50); // make sure the form is hidden before capture
   Grab(image1.Picture.Bitmap, GTSCREEN);
   image1.Width := image1.Picture.Bitmap.Width;
   image1.Height := image1.Picture.Bitmap.Height;
   DoSelect := true;
   MaximizeForm;
   Form1.FormStyle := fsStayOnTop;
   Form1.Show;
end;

end.
